@@ -34,9 +34,11 @@ Synthetic Initialization Vector (SIV) mode.
 
 __all__ = ['SivMode']
 
-from binascii import hexlify
+from binascii import hexlify, unhexlify
 
-from Cryptodome.Util.py3compat import byte_string, bord, unhexlify, _copy_bytes
+from Cryptodome.Util.py3compat import bord, _copy_bytes
+
+from Cryptodome.Util._raw_api import is_buffer
 
 from Cryptodome.Util.number import long_to_bytes, bytes_to_long
 from Cryptodome.Protocol.KDF import _S2V
@@ -99,8 +101,8 @@ class SivMode(object):
             raise ValueError("Incorrect key length (%d bytes)" % len(key))
 
         if nonce is not None:
-            if isinstance(nonce, unicode):
-                raise TypeError("When provided, the nonce must be a byte string")
+            if not is_buffer(nonce):
+                raise TypeError("When provided, the nonce must be bytes, bytearray or memoryview")
 
             if len(nonce) == 0:
                 raise ValueError("When provided, the nonce must be non-empty")
@@ -269,17 +271,24 @@ class SivMode(object):
 
         self.verify(unhexlify(hex_mac_tag))
 
-    def encrypt_and_digest(self, plaintext):
+    def encrypt_and_digest(self, plaintext, output=None):
         """Perform encrypt() and digest() in one step.
 
         :Parameters:
           plaintext : bytes/bytearray/memoryview
             The piece of data to encrypt.
+        :Keywords:
+          output : bytearray/memoryview
+            The location where the ciphertext must be written to.
+            If ``None``, the ciphertext is returned.
         :Return:
-            a tuple with two byte strings:
+            a tuple with two items:
 
-            - the encrypted data
-            - the MAC
+            - the ciphertext, as ``bytes``
+            - the MAC tag, as ``bytes``
+
+            The first item becomes ``None`` when the ``output`` parameter
+            specified a location for the result.
         """
         
         if self.encrypt not in self._next:
@@ -296,9 +305,9 @@ class SivMode(object):
         
         cipher = self._create_ctr_cipher(self._mac_tag)
 
-        return cipher.encrypt(plaintext), self._mac_tag
+        return cipher.encrypt(plaintext, output=output), self._mac_tag
 
-    def decrypt_and_verify(self, ciphertext, mac_tag):
+    def decrypt_and_verify(self, ciphertext, mac_tag, output=None):
         """Perform decryption and verification in one step.
 
         A cipher object is stateful: once you have decrypted a message
@@ -316,8 +325,12 @@ class SivMode(object):
             It can be of any length.
           mac_tag : bytes/bytearray/memoryview
             This is the *binary* MAC, as received from the sender.
-
-        :Return: the decrypted data (byte string).
+        :Keywords:
+          output : bytearray/memoryview
+            The location where the plaintext must be written to.
+            If ``None``, the plaintext is returned.
+        :Return: the plaintext as ``bytes`` or ``None`` when the ``output``
+            parameter specified a location for the result.
         :Raises ValueError:
             if the MAC does not match. The message has been tampered with
             or the key is incorrect.
@@ -331,11 +344,11 @@ class SivMode(object):
         # Take the MAC and start the cipher for decryption
         self._cipher = self._create_ctr_cipher(mac_tag)
 
-        plaintext = self._cipher.decrypt(ciphertext)
+        plaintext = self._cipher.decrypt(ciphertext, output=output)
 
         if hasattr(self, 'nonce'):
             self._kdf.update(self.nonce)
-        self._kdf.update(plaintext)
+        self._kdf.update(plaintext if output is None else output)
         self.verify(mac_tag)
         
         return plaintext
@@ -371,7 +384,7 @@ def _create_siv_cipher(factory, **kwargs):
 
     try:
         key = kwargs.pop("key")
-    except KeyError, e:
+    except KeyError as e:
         raise TypeError("Missing parameter: " + str(e))
 
     nonce = kwargs.pop("nonce", None)
