@@ -77,7 +77,7 @@ class DerObject(object):
         This class should never be directly instantiated.
         """
 
-        def __init__(self, asn1Id=None, payload=b(''), implicit=None,
+        def __init__(self, asn1Id=None, payload=b'', implicit=None,
                      constructed=False, explicit=None):
                 """Initialize the DER object according to a specific ASN.1 type.
 
@@ -185,14 +185,16 @@ class DerObject(object):
                 """Decode DER length octets from a file."""
 
                 length = s.read_byte()
-                if length <= 127:
-                        return length
-                payloadLength = bytes_to_long(s.read(length & 0x7F))
-                # According to DER (but not BER) the long form is used
-                # only when the length doesn't fit into 7 bits.
-                if payloadLength <= 127:
-                        raise ValueError("Not a DER length tag (but still valid BER).")
-                return payloadLength
+
+                if length > 127:
+                    encoded_length = s.read(length & 0x7F)
+                    if bord(encoded_length[0]) == 0:
+                        raise ValueError("Invalid DER: length has leading zero")
+                    length = bytes_to_long(encoded_length)
+                    if length <= 127:
+                        raise ValueError("Invalid DER: length in long form but smaller than 128")
+
+                return length
 
         def decode(self, der_encoded, strict=False):
                 """Decode a complete DER element, and re-initializes this
@@ -284,7 +286,7 @@ class DerInteger(DerObject):
                     It overrides the universal tag for INTEGER (2).
                 """
 
-                DerObject.__init__(self, 0x02, b(''), implicit,
+                DerObject.__init__(self, 0x02, b'', implicit,
                                    False, explicit)
                 self.value = value  # The integer value
 
@@ -293,7 +295,7 @@ class DerInteger(DerObject):
                 binary string."""
 
                 number = self.value
-                self.payload = b('')
+                self.payload = b''
                 while True:
                     self.payload = bchr(int(number & 255)) + self.payload
                     if 128 <= number <= 255:
@@ -395,7 +397,7 @@ class DerSequence(DerObject):
                     It overrides the universal tag for SEQUENCE (16).
                 """
 
-                DerObject.__init__(self, 0x10, b(''), implicit, True)
+                DerObject.__init__(self, 0x10, b'', implicit, True)
                 if startSeq is None:
                     self._seq = []
                 else:
@@ -440,9 +442,9 @@ class DerSequence(DerObject):
                   only_non_negative (boolean):
                     If ``True``, negative integers are not counted in.
                 """
-                def _is_number2(x):
-                    return _is_number(x, only_non_negative)
-                return len(filter(_is_number2, self._seq))
+                
+                items = [x for x in self._seq if _is_number(x, only_non_negative)]
+                return len(items)
 
         def hasOnlyInts(self, only_non_negative=True):
                 """Return ``True`` if all items in this sequence are integers
@@ -465,7 +467,7 @@ class DerSequence(DerObject):
                   ValueError: if some elements in the sequence are neither integers
                               nor byte strings.
                 """
-                self.payload = b('')
+                self.payload = b''
                 for item in self._seq:
                     if byte_string(item):
                         self.payload += item
@@ -475,7 +477,7 @@ class DerSequence(DerObject):
                         self.payload += item.encode()
                 return DerObject.encode(self)
 
-        def decode(self, der_encoded, nr_elements=None, only_ints_expected=False, strict=False):
+        def decode(self, der_encoded, strict=False, nr_elements=None, only_ints_expected=False):
                 """Decode a complete DER SEQUENCE, and re-initializes this
                 object with it.
 
@@ -486,7 +488,7 @@ class DerSequence(DerObject):
                     The number of members the SEQUENCE can have
                   only_ints_expected (boolean):
                     Whether the SEQUENCE is expected to contain only integers.
-                  struct (boolean):
+                  strict (boolean):
                     Whether decoding must check for strict DER compliancy.
 
                 Raises:
@@ -572,7 +574,7 @@ class DerOctetString(DerObject):
     :vartype payload: byte string
     """
 
-    def __init__(self, value=b(''), implicit=None):
+    def __init__(self, value=b'', implicit=None):
         """Initialize the DER object as an OCTET STRING.
 
         :Parameters:
@@ -593,7 +595,7 @@ class DerNull(DerObject):
     def __init__(self):
         """Initialize the DER object as a NULL."""
 
-        DerObject.__init__(self, 0x05, b(''), None, False)
+        DerObject.__init__(self, 0x05, b'', None, False)
 
 
 class DerObjectId(DerObject):
@@ -638,39 +640,44 @@ class DerObjectId(DerObject):
           explicit : integer
             The EXPLICIT tag to use for the encoded object.
         """
-        DerObject.__init__(self, 0x06, b(''), implicit, False, explicit)
+        DerObject.__init__(self, 0x06, b'', implicit, False, explicit)
         self.value = value
 
     def encode(self):
         """Return the DER OBJECT ID, fully encoded as a
         binary string."""
 
-        comps = map(int, self.value.split("."))
+        comps = [int(x) for x in self.value.split(".")]
         if len(comps) < 2:
             raise ValueError("Not a valid Object Identifier string")
         self.payload = bchr(40*comps[0]+comps[1])
         for v in comps[2:]:
-            enc = []
-            while v:
-                enc.insert(0, (v & 0x7F) | 0x80)
-                v >>= 7
-            enc[-1] &= 0x7F
-            self.payload += b('').join(map(bchr, enc))
+            if v == 0:
+                enc = [0]
+            else:
+                enc = []
+                while v:
+                    enc.insert(0, (v & 0x7F) | 0x80)
+                    v >>= 7
+                enc[-1] &= 0x7F
+            self.payload += b''.join([bchr(x) for x in enc])
         return DerObject.encode(self)
 
-    def decode(self, der_encoded):
+    def decode(self, der_encoded, strict=False):
         """Decode a complete DER OBJECT ID, and re-initializes this
         object with it.
 
         Args:
             der_encoded (byte string):
                 A complete DER OBJECT ID.
+            strict (boolean):
+                Whether decoding must check for strict DER compliancy.
 
         Raises:
             ValueError: in case of parsing errors.
         """
 
-        return DerObject.decode(self, der_encoded)
+        return DerObject.decode(self, der_encoded, strict)
 
     def _decodeFromStream(self, s, strict):
         """Decode a complete DER OBJECT ID from a file."""
@@ -680,7 +687,7 @@ class DerObjectId(DerObject):
 
         # Derive self.value from self.payload
         p = BytesIO_EOF(self.payload)
-        comps = list(map(str, divmod(p.read_byte(), 40)))
+        comps = [str(x) for x in divmod(p.read_byte(), 40)]
         v = 0
         while p.remaining_data():
             c = p.read_byte()
@@ -721,7 +728,7 @@ class DerBitString(DerObject):
     :vartype value: byte string
     """
 
-    def __init__(self, value=b(''), implicit=None, explicit=None):
+    def __init__(self, value=b'', implicit=None, explicit=None):
         """Initialize the DER object as a BIT STRING.
 
         :Parameters:
@@ -734,7 +741,7 @@ class DerBitString(DerObject):
           explicit : integer
             The EXPLICIT tag to use for the encoded object.
         """
-        DerObject.__init__(self, 0x03, b(''), implicit, False, explicit)
+        DerObject.__init__(self, 0x03, b'', implicit, False, explicit)
 
         # The bitstring value (packed)
         if isinstance(value, DerObject):
@@ -747,21 +754,23 @@ class DerBitString(DerObject):
         binary string."""
 
         # Add padding count byte
-        self.payload = b('\x00') + self.value
+        self.payload = b'\x00' + self.value
         return DerObject.encode(self)
 
-    def decode(self, der_encoded):
+    def decode(self, der_encoded, strict=False):
         """Decode a complete DER BIT STRING, and re-initializes this
         object with it.
 
         Args:
             der_encoded (byte string): a complete DER BIT STRING.
+            strict (boolean):
+                Whether decoding must check for strict DER compliancy.
 
         Raises:
             ValueError: in case of parsing errors.
         """
 
-        return DerObject.decode(self, der_encoded)
+        return DerObject.decode(self, der_encoded, strict)
 
     def _decodeFromStream(self, s, strict):
         """Decode a complete DER BIT STRING DER from a file."""
@@ -773,7 +782,7 @@ class DerBitString(DerObject):
             raise ValueError("Not a valid BIT STRING")
 
         # Fill-up self.value
-        self.value = b('')
+        self.value = b''
         # Remove padding count byte
         if self.payload:
             self.value = self.payload[1:]
@@ -816,7 +825,7 @@ class DerSetOf(DerObject):
             The IMPLICIT tag to use for the encoded object.
             It overrides the universal tag for SET OF (17).
         """
-        DerObject.__init__(self, 0x11, b(''), implicit, True)
+        DerObject.__init__(self, 0x11, b'', implicit, True)
         self._seq = []
 
         # All elements must be of the same type (and therefore have the
@@ -860,7 +869,7 @@ class DerSetOf(DerObject):
         if elem not in self._seq:
             self._seq.append(elem)
 
-    def decode(self, der_encoded):
+    def decode(self, der_encoded, strict=False):
         """Decode a complete SET OF DER element, and re-initializes this
         object with it.
 
@@ -869,12 +878,14 @@ class DerSetOf(DerObject):
 
         Args:
             der_encoded (byte string): a complete DER BIT SET OF.
+            strict (boolean):
+                Whether decoding must check for strict DER compliancy.
 
         Raises:
             ValueError: in case of parsing errors.
         """
 
-        return DerObject.decode(self, der_encoded)
+        return DerObject.decode(self, der_encoded, strict)
 
     def _decodeFromStream(self, s, strict):
         """Decode a complete DER SET OF from a file."""
@@ -925,5 +936,5 @@ class DerSetOf(DerObject):
                 bys = item
             ordered.append(bys)
         ordered.sort()
-        self.payload = b('').join(ordered)
+        self.payload = b''.join(ordered)
         return DerObject.encode(self)
